/*
 Copyright (c) 2015 WiseCRM.com. All rights reserved.
 WiseCRM.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.wisecloudcrm.server.openapi.sdk;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;
import java.util.zip.GZIPInputStream;

import org.apache.commons.httpclient.Header;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.MultiThreadedHttpConnectionManager;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.StringRequestEntity;
import org.apache.commons.httpclient.params.HttpClientParams;
import org.apache.commons.httpclient.params.HttpConnectionManagerParams;
import org.apache.commons.httpclient.params.HttpMethodParams;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;

/**
 * @author Zhao Fangfang
 * @since 2016-3-6
 * @version $Id$
 */
public class OpenApiSDK {
	
	private static final Log LOG = LogFactory.getLog(OpenApiSDK.class);

	// ޸ΪӦ
	public static final String API_URL = "http://d.crm.goutonghui.com/gw/";
	public static final String APP_ID = "";
	public static final String APP_KEY = "";
	
	public static final String SIGN_TYPE = "MD5";
	public static final String ENCODING = "utf-8";
	
	private static HttpClient httpClient = null;
	
	/**
	 *  HTTP/GET API 
	 * 
	 * @param apiUrl
	 * @return
	 */
	public static JSONObject apiGet(String apiUrl) {
		HttpMethod method = new GetMethod(apiUrl);
		try {
			int status = getHttpClient().executeMethod(method);
			if (status != 200) {
				LOG.error("HTTP/GET ش״̬: " + status + ", URL: " + apiUrl);
				return null;
			}
			
			String result = readResponse(method);
			if (result == null) {
				LOG.error("HTTP/GET ؿ. URL: " + apiUrl);
				return null;
			}
			return JSON.parseObject(result);
		} catch (Exception e) {
			LOG.error("HTTP/GET ִ", e);
			return null;
		}
	}
	
	/**
	 *  HTTP/POST API 
	 * 
	 * @param apiUrl
	 * @param postdata
	 * @return
	 */
	public static JSONObject apiPost(String apiUrl, String postdata) {
		PostMethod method = new PostMethod(apiUrl);
		try {
			if (postdata != null) {
				method.setRequestEntity(new StringRequestEntity(postdata, "application/json", ENCODING));
			}
			
			int status = getHttpClient().executeMethod(method);
			if (status != 200) {
				LOG.error("HTTP/POST ش״̬: " + status + ",  URL: " + apiUrl);
				return null;
			}
			
			String result = readResponse(method);
			if (result == null) {
				LOG.error("HTTP/POST ˿.  URL: " + apiUrl);
				return null;
			}
			return JSON.parseObject(result);
		} catch (Exception e) {
			LOG.error("HTTP/POST ִ", e);
			return null;
		}
	}
	
	/**
	 * ɴǩ API  URL
	 * 
	 * @param bizParams
	 * @return
	 */
	public static String signUrl(String apiName, Map<String, String> bizParams) {
		return signUrl(API_URL, apiName, bizParams);
	}
	
	/**
	 * ɴǩ API  URL
	 * 
	 * @param apiUrl
	 * @param apiName
	 * @param bizParams
	 * @return
	 */
	public static String signUrl(String apiUrl, String apiName, Map<String, String> bizParams) {
		// ǩ
		bizParams.put("appid", APP_ID);
		bizParams.put("sign_type", SIGN_TYPE);
		bizParams.put("timestamp", System.currentTimeMillis() + "");
		
		String sign = sign(bizParams);
		bizParams.put("sign", sign);
		
	    apiUrl = apiUrl + apiName + "?";
		for (Map.Entry<String, String> e : bizParams.entrySet()) {
			apiUrl += e.getKey() + "=" + urlEncode(e.getValue()) + "&";
		}
		apiUrl = apiUrl.substring(0, apiUrl.length() - 1);  // remove &
		return apiUrl;
	}
	
	/**
	 * ǩַ
	 * 
	 * @param params
	 * @return
	 */
	public static String sign(Map<String, String> params) {
		Map<String, String> tMap = new TreeMap<String, String>(params);
		StringBuffer sign2str = new StringBuffer();
        for (Map.Entry<String, String> e : tMap.entrySet()) {
        	if (StringUtils.isBlank(e.getValue())) {
        		continue;
        	}
        	sign2str.append(e.getKey() + "=" + e.getValue()).append("&");
        }
        sign2str.append("appkey=" + APP_KEY);
		
        String sign2sign = null;
        if ("MD5".equalsIgnoreCase(SIGN_TYPE)) {
        	sign2sign = toMD5(sign2str.toString().getBytes());
        } else if ("SHA1".equalsIgnoreCase(SIGN_TYPE)) {
        	sign2sign = toSHA1(sign2str.toString().getBytes());
        }
        return sign2sign;
	}
	
	/**
	 *  API ҪǼȨģ
	 * 
	 * @return
	 */
	public static Map<String, String> baseApiParams() {
		Map<String, String> params = new HashMap<String, String>();
		params.put("appid", APP_ID);
		params.put("sign_type", SIGN_TYPE);
		params.put("timestamp", System.currentTimeMillis() + "");
		return params;
	}
	
	/**
	 *  API ҪǼȨģ
	 * 
	 * @param userId
	 * @return
	 */
	public static Map<String, String> baseApiParams(Serializable userId) {
		Map<String, String> params = baseApiParams();
		params.put("op_user", userId.toString());
		return params;
	}
	
	// ----------- 
	
	/**
	 * ȡ HttpClient
	 * 
	 * @return
	 */
	public static HttpClient getHttpClient() {
		if (httpClient != null) {
			return httpClient;
		}
		
		HttpClientParams clientParams = new HttpClientParams();
		clientParams.setConnectionManagerTimeout(5 * 1000);
		clientParams.setSoTimeout(15 * 1000);
		clientParams.setParameter(HttpMethodParams.HTTP_CONTENT_CHARSET, ENCODING);
		clientParams.setConnectionManagerClass(MultiThreadedHttpConnectionManager.class);
		httpClient = new HttpClient(clientParams);

		HttpConnectionManagerParams connectionParams = httpClient.getHttpConnectionManager().getParams();
		connectionParams.setDefaultMaxConnectionsPerHost(20);
		connectionParams.setMaxTotalConnections(200);
		connectionParams.setConnectionTimeout(15 * 1000);
		return httpClient;
	}
	
	/**
	 * ȡ
	 * 
	 * @param method
	 * @return
	 * @throws IOException
	 */
	public static String readResponse(HttpMethod method) throws IOException {
		String responseText = null;
		BufferedReader reader = null;
		try {
			InputStream stream = method.getResponseBodyAsStream();
			if (stream.available() > -1) {
				Header encoding = method.getResponseHeader("Content-Encoding");
				if (encoding != null && encoding.getValue().toUpperCase().contains("GZIP")) {
					stream = new GZIPInputStream(stream);
				}
				String enc = StringUtils.defaultIfEmpty(
						method.getParams().getContentCharset(), ENCODING);
				reader = new BufferedReader(new InputStreamReader(stream, enc));
				
				StringBuffer sb = new StringBuffer();
				char[] buffer = new char[1024 * 4];
				int read = -1;
				while ((read = reader.read(buffer)) > 0) {
					char[] chunk = new char[read];
					System.arraycopy(buffer, 0, chunk, 0, read);
					sb.append(chunk);
				}
				responseText = sb.toString();
			}
			
		} catch (IOException ex) {
			throw ex;
		} finally {
			try {
				reader.close();
			} catch (Exception ex){}
			method.releaseConnection();
		}
		return responseText;
	}
	
	/**
	 * MD5
	 * 
	 * @param input
	 * @return
	 */
	public static String toMD5(byte[] input) {
		MessageDigest digest = null;
		try {
			digest = MessageDigest.getInstance("MD5");
		} catch (NoSuchAlgorithmException e) {
			throw new RuntimeException(e);
		}
		digest.update(input);
		return toHexString(digest.digest());
	}

	/**
	 * SHA1
	 * 
	 * @param input
	 * @return
	 */
	public static String toSHA1(byte[] input) {
		MessageDigest digest = null;
		try {
			digest = MessageDigest.getInstance("SHA1");
		} catch (NoSuchAlgorithmException e) {
			throw new RuntimeException(e);
		}
		digest.update(input);
		return toHexString(digest.digest());
	}
	
	/**
	 * URL
	 * 
	 * @param text
	 * @return
	 */
	public static String urlEncode(String text) {
		if (text == null) {
			return null;
		}
		try {
			return URLEncoder.encode(text, ENCODING);
		} catch (UnsupportedEncodingException e) {
			throw new RuntimeException("Exception encoding URL string: " + e);
		}
	}
	
	/**
	 * @param data
	 * @return
	 */
	private static String toHexString(byte[] data) {
		StringBuilder sb = new StringBuilder();
		for (byte b : data) {
			sb.append(String.format("%02x", b));
		}
		return sb.toString();
	}
	
	// 
	public static void main(String[] args) {
		Map<String, String> params = OpenApiSDK.baseApiParams();
		params.put("user", "xx");
		params.put("password", toMD5("xx".getBytes()));
		
		String apiUrl = OpenApiSDK.signUrl("user/login-token", params);
		JSONObject r = OpenApiSDK.apiGet(apiUrl);
		System.out.println(r);
	}
}
